"""
This file is only here for backwards compatibility and there is a
good chance it will disppear soon. Please make use of:
:mod:`easymode.admin.models.fields` and :mod:`easymode.admin.forms.fields`
instead.
"""

from easymode.admin.forms.fields import *
from easymode.admin.models.fields import *


import os
import sys

from django.conf import settings

from easymode import i18n
from easymode.i18n import meta


__all__ = ('I18n',)

class I18n(object):
    """
    Internationalise a model class.
    
    use like this:
    
    >>> from django.db import models
    >>> from easymode.i18n.decorators import I18n
    >>> 
    >>> @I18n('iamatranslatedfield')
    >>> class Bla(models.Model):
    >>>     iamafield = models.CharField(max_length=255)
    >>>     iamatranslatedfield = models.CharField(max_length=255)
    
    Now ``iamatranslatedfield`` it's value can vary by language.
    """
    def __init__(self, *localized_fields):
        """initialize the decorator"""
        self.localized_fields = localized_fields
        
    def __call__(self, cls):
        """Executes the decorator on the cls."""
        model_dir = os.path.dirname(sys.modules[cls.__module__].__file__) + getattr(settings, 'LOCALE_POSTFIX', '')
        cls = meta.localize_fields(cls, self.localized_fields)
        if getattr(settings, 'AUTO_CATALOG', False):
            i18n.register(cls, getattr(settings, 'LOCALE_DIR', None) or model_dir )
        
        # add permission for editing the untranslated fields in this model
        cls._meta.permissions.append(
            ("can_edit_untranslated_fields_of_%s" % cls.__name__.lower(), 
            "Can edit untranslated fields")
        )

        return cls



"""
Gettext related functionality.
"""
import os
import re
import subprocess
import codecs
import logging
import StringIO
import unicodedata

from datetime import datetime

from django.conf import settings
from django.core.management.base import CommandError
from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import templatize, to_locale, get_language
from django.utils import tzinfo
from django.template.loader import render_to_string

import easymode.tree.introspection
from easymode.utils import mutex
from easymode.utils.languagecode import get_language_codes
from easymode.utils import polibext

if 'rosetta' in settings.INSTALLED_APPS:
    from rosetta import polib
else:
    try:
        import polib
    except ImportError:
        raise ImproperlyConfigured(
            """
            Please install either django-rosetta: 
            http://code.google.com/p/django-rosetta/
            or polib:
            http://bitbucket.org/izi/polib/src/
            otherwise easymode.i18n.gettext won't work"""
        )

__all__ = ('MakeModelMessages', 'XGETTEXT_REENCODES_UTF8')


source_re = re.compile(r'#: .*?:(\d+)', re.UNICODE)

# check if we have a crap version of xgettext
XGETTEXT_REENCODES_UTF8 = False
p = subprocess.Popen('xgettext --version', shell=True, stdin=subprocess.PIPE,
    stdout=subprocess.PIPE, stderr=subprocess.PIPE)
(msg, err) = p.communicate()
match = re.search(r'(?P<major>\d+)\.(?P<minor>\d+)', msg)
if match:
    xversion = (int(match.group('major')), int(match.group('minor')))
    if xversion < (0, 15):
        XGETTEXT_REENCODES_UTF8 = True
 
 
class MakeModelMessages(object):
    """
    Class that can be set as a post_save signal handler.
    If done so, it will create and update a django.po file
    for each language in settings. But only if the model is
    create newly, not when it is updated manually.    
    """

    def __init__(self, location=None, sender=None):

        if location is None:
            self.location = (hasattr(settings, 'LOCALE_DIR') and \
                settings.LOCALE_DIR) or settings.PROJECT_DIR
        elif os.path.isdir(location) is False and os.path.exists(location):
            self.location = os.path.dirname(location)
        else:
            self.location = location
            
        self.sender = sender

    
    def __call__(self, instance, sender=None, **kwargs):
        """
        Handler for the post_save signal.
        Will create a django.po file in each language,
        in the directory specified by self.location.
        """

        # no need to create translation files if not in the master site
        if not getattr(settings, 'MASTER_SITE', False):
            return
            
        if hasattr(instance, 'language') and instance.language != settings.LANGUAGE_CODE:
            return
        
        if get_language() != settings.LANGUAGE_CODE:
            return

        po_form = self.poify(instance)
        if po_form is not None:
            # for each language create a po file
            for language in get_language_codes():
                
                locale_dir = os.path.join( self.location , 'locale',
                    to_locale(language), 'LC_MESSAGES')
                locale_file = os.path.join( locale_dir, 'django.po')

                # create locale dir if not available
                if not os.path.isdir(locale_dir):
                    os.makedirs(locale_dir)
                    
                # acquire exclusive lock this should only be run
                # by one process at a time                
                with mutex(max_wait=30):
                    # write proper header if the file is new
                    # this header contains the character set.
                    write_header = not os.path.exists(locale_file)
                    
                    if write_header: # write header and po stuffz0r
                        with codecs.open(locale_file, 'w', 'utf-8') as fp:
                            fp.write(polibext.po_to_unicode(po_form))
                            
                    else: 
                        #merge existing translation with the new one
                        msg = self.msgmerge(locale_file, polibext.po_to_unicode(po_form).encode('utf-8'))
                        if msg:
                            if len(po_form):
                                # add the new entries to the po file
                                with codecs.open(locale_file, 'a', 'utf-8') as fp:
                                    for entry in po_form: # this avoids printing the header
                                        fp.write(u"\n")
                                        fp.write(polibext.po_to_unicode(entry))
                                        
                                # filter away duplicates  
                                msg = self.msguniq(locale_file)
                                # save entire po file once more and overwrite with filtered shizzle
                                with open(locale_file, 'w') as fp:
                                    fp.write(msg)                                

    
    def poify(self, model):
        """turn a django model into a po file."""
        if not hasattr(model, 'localized_fields'):
            return None
        
        # create po stream with header
        po_stream = polibext.PoStream(StringIO.StringIO(self.po_header)).parse()
        
        for (name, field) in easymode.tree.introspection.get_default_field_descriptors(model):
            occurrence = u"%s.%s.%s" % (model._meta.app_label, model.__class__.__name__, name)
            value = field.value_to_string(model)
            
            # only add empty strings
            if value != "":                
                entry = polib.POEntry(msgid=value, occurrences=[(occurrence, model.pk)])
                # make sure no duplicate entries in the po_stream
                existing_entry = po_stream.find(entry.msgid)
                if existing_entry is None:
                    po_stream.append(entry)
                else:
                    # no really, existing_entry.merge does not merge the occurrences.
                    existing_entry.occurrences += entry.occurrences
        
        return po_stream
    

    def xgettext(self, template):
        """Extracts to be translated strings from template and turns it into po format."""
        
        cmd = 'xgettext -d django -L Python --keyword=gettext_noop \
            --keyword=gettext_lazy --keyword=ngettext_lazy:1,2 --from-code=UTF-8 \
            --output=- -'

        p = subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE,
            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        (msg, err) = p.communicate(input=templatize(template))
        
        if err:
            # dont raise exception, some stuff in stderr are just warmings
            logging.warning(err)

        if XGETTEXT_REENCODES_UTF8:
            return msg.decode('utf-8').encode('iso-8859-1')
                    
        return msg

    
    def po_stream(self, po_string):
        """make a po stream object from a po_string"""
        return polibext.PoStream(StringIO.StringIO(po_string)).parse()


    def msgmerge(self, locale_file, po_string):
        """
        Runs msgmerge on a locale_file and po_string
        """
        
        cmd = "msgmerge -q %s -" % locale_file
        p = subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE,
            stdout=subprocess.PIPE, stderr=subprocess.PIPE) 
        (msg, err) = p.communicate(input=po_string)
        
        if err:
            # dont raise exception, some stuff in stderr are just warmings
            logging.warning("%s \nfile: %s\npostring: %s" % (err, locale_file, po_string))
        
        return msg

        
    def msguniq(self, locale_file):
        """
        run msgunique on the locale_file
        """
        
        # group related language strings together.
        # except if no real entries where written or the header will be removed.
        p = subprocess.Popen('msguniq --to-code=utf-8 %s' % (locale_file,),
            shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE )
        (msg, err) = p.communicate()
    
        if err:
            # raise exception, none of the stuff in stderr are just warmings
            logging.error(err)
            try:
                err = unicodedata.normalize('NFKD', err.decode('utf-8')).encode('ascii','ignore')
            except UnicodeError:
                err = "can not decode error message"
            raise CommandError(
                u"error happened while running msguniq on: %s %s" % \
                    (locale_file, err)
            )

        return msg
        
            
    @property
    def po_header(self):
        localtime = tzinfo.LocalTimezone( datetime.now() )
        now = datetime.now(localtime).isoformat()
        po_header = render_to_string('i18n/po_header.txt', { 
            'creation_date': now,
            'revision_date': now,
            'last_translator' : u'anonymous',
            'last_translator_email' : u'anonymous@example.com',
            'language_team' : u'anonymous',
            'language_team_email': u'anonymous@example.com',
        })

        return po_header



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.conf import settings


__all__ = ('EasyPublisherModel', 'EasyPublisherModelManager')

PUBLICATION_STATUSSES = (
    ('published', _('Published')),
    ('draft', _('Draft')),
    ('needs_work', _('Needs work')),
    ('updated', _('Updated')),
    ('declined', _('Declined')),
)

class EasyPublisherModelManager(models.Manager):
    """
    A default manager that will only return published items.
    """
    def get_query_set(self):
        return super(EasyPublisherModelManager, self).get_query_set().filter(published=True)


class EasyPublisherModel(models.Model):
    """
    An abstract baseclass for your model which allows
    easypublisher to be sure that the model has a *published* property.
    """
    published = models.BooleanField(_('Published'), default=True)
    
    class Meta:
        abstract = True
    

class EasyPublisherMetaData(models.Model):
    """
    A model for adding metadata to a reversion version.
    """
    
    class Meta:
        permissions = (('can_approve_for_publication', 'Can approve for publication'),)
    
    revision = models.ForeignKey("reversion.Revision")
    status = models.CharField(_('Publication Status'), max_length=10, choices=PUBLICATION_STATUSSES, default='draft')
    language = models.CharField(_('Language'), max_length=5, choices=settings.LANGUAGES, default=settings.LANGUAGE_CODE)
    
    def __unicode__(self):
        return u"revision: %s status: %s language: %s" % (self.revision, self.status, self.language)



import re
import warnings

from django.test import TestCase
from django.utils import translation
from django.forms.models import model_to_dict
from django.core.urlresolvers import reverse

from easymode.tests.testcases import initdb
from easymode.tests.models import TestL10nModel
from easymode import tree

__all__ = ('TestDiocoreFields',)

@initdb
class TestDiocoreFields(TestCase):
    """docstring for TestDiocoreFields"""
    
    fixtures = ['auth-user', 'tests-testl10nmodel']
        
    def test_diocore_fields_are_special(self):
        """Diocore fields should have a font attribute after serialization"""
        i = TestL10nModel.objects.get(pk=1)
        xml = tree.xml(i)

        title_has_font = re.search(r'field font="arial" type="CharField" name="title"', xml)
        body_has_font = re.search(r'field font="arial" type="TextField" name="body"', xml)

        self.assertTrue(title_has_font)
        self.assertTrue(body_has_font)
    
    def test_diocore_fields_strip_cariage_returns(self):
        "Diocore fields should strip cariage returns from the input"
                
        h = {
            'title': "Ik ben de groot moeftie van oeral",
            'description': "Ik weet\r weinig tot niks van brei \rcursussen",
            'body':"Het zou kunnen dat\r linksaf\r slaan hier een goede keuze is.",
            'price':34.0,
        }
        
        if self.client.login(username='admin', password='admin'):
            
            add_view_url = reverse("admin:tests_testl10nmodel_add")            
            response = self.client.post(add_view_url, h)

            i = TestL10nModel.objects.get(pk=3)

            assert(i.title == 'Ik ben de groot moeftie van oeral')
            assert(i.description == "Ik weet weinig tot niks van brei cursussen")
            assert(i.body == "Het zou kunnen dat linksaf slaan hier een goede keuze is.")
        else:
            self.fail("can not login to backend using admin/admin")



from django.conf import settings
from django.template import TemplateDoesNotExist
from django.template.loader import template_source_loaders, find_template_loader

def find_template_path(name):
    """
    Same as :func:`django.template.loader.find_template`, but it only returns the path
    to the template file.
    """
    global template_source_loaders
    if template_source_loaders is None:
        loaders = []
        for loader_name in settings.TEMPLATE_LOADERS:
            loader = find_template_loader(loader_name)
            if loader is not None:
                loaders.append(loader)
        template_source_loaders = tuple(loaders)
    for loader in template_source_loaders:
        try:
            _, display_name = loader(name)
            if display_name:
                return display_name
        except TemplateDoesNotExist:
            pass
    raise TemplateDoesNotExist(name)



from django.contrib.contenttypes.generic import BaseGenericInlineFormSet
from django.contrib.contenttypes.models import ContentType
from django.forms.models import save_instance


__all__ = ('LocalizableGenericInlineFormSet',)


class LocalizableGenericInlineFormSet(BaseGenericInlineFormSet):
    """
    Generic inline formset that is a bit less dumb about
    fields that are not model fields but are on the form anyway.
    """

    def save_new(self, form, commit=True):
        # Avoid a circular import.
        kwargs = {
            self.ct_field.get_attname(): ContentType.objects.get_for_model(self.instance).pk,
            self.ct_fk_field.get_attname(): self.instance.pk,
        }
        
        # try to add all data that can be used to initialize the model.
        cleaned_data = form.cleaned_data
        initial_data = dict([(key, cleaned_data[key]) for key  in cleaned_data.keys() if key in dir(self.model)])
        initial_data.update(kwargs)
        new_obj = self.model(**initial_data)
        return save_instance(form, new_obj, commit=commit)    


from django.contrib import admin

from easymode.easypublisher.admin import EasyPublisher
from easymode.i18n.admin.decorators import L10n
from easymode.tests import models as test_models
from easymode.tests import forms as test_forms
from easymode.tree.admin import relation


class TestSubModelInline(admin.StackedInline):
    """docstring for TestSubModelInline"""
    model = test_models.TestSubModel
    extra = 1
    order_field = 'order'


class TestModelAdmin(relation.ForeignKeyAwareModelAdmin):
    """docstring for TestModelAdmin"""

    inlines = [TestSubModelInline]
    
    children = [test_models.TestSubModel,test_models.TestSecondSubmodel]
    invisible_in_admin = False


class SelfAwareTestModelAdmin(relation.ForeignKeyAwareModelAdmin):
    model = test_models.TestModel

@L10n(test_models.TestL10nModel)
class TestL10nModelAdmin(EasyPublisher):
    pass

class TestEasypublisherInline(admin.StackedInline):
    model = test_models.TestEasypublisherRelatedModel

class TestEasypublisherAdmin(EasyPublisher):
    inlines = [TestEasypublisherInline]

@L10n
class FormTestAdmin(admin.ModelAdmin):
    form = test_forms.OverrideForm
    model = test_models.FormTestModel

admin.site.register(test_models.TestModel, TestModelAdmin)
admin.site.register(test_models.TestSubModel)
admin.site.register(test_models.TestSecondSubmodel)
admin.site.register(test_models.TestL10nModel, TestL10nModelAdmin)
admin.site.register(test_models.TestEasypublisherModel, TestEasypublisherAdmin)
admin.site.register(test_models.ManagerErrorModel)
admin.site.register(test_models.FormTestModel, FormTestAdmin)


"""
functionality for finding inverse foreign key relations in model classes
"""
from django.contrib.contenttypes.generic import ReverseGenericRelatedObjectsDescriptor
from django.db.models.base import ModelBase
from django.db.models.fields.related import ForeignRelatedObjectsDescriptor, SingleRelatedObjectDescriptor

from easymode.i18n.meta import DefaultFieldDescriptor


INTROSPECTION_ERROR = """
%s

Easymode caught an AttributeError while trying
to inspect %s looking for %s.

Please report to easymode@librelist.com.
"""

def _get_members_of_type(obj, member_type):
    """
    Finds members of a certain type in obj.

    :param obj: A model instance or class.
    :param member_type: The type of the menber we are trying to find.
    :rtype: A :class:`list` of ``member_type`` found in ``obj``
    """
    
    if not issubclass(type(obj), ModelBase):
        obj = obj.__class__
        
    key_hash = []
    for key in dir(obj):
        try:
            attr = getattr(obj, key)
        except AttributeError as e:
            try:
                attr = obj.__dict__[key]
            except KeyError:
                raise AttributeError(INTROSPECTION_ERROR % (e, obj, member_type))

        if type(attr) is member_type:
            key_hash.append((key, attr))
    
    return key_hash

def get_foreign_key_desciptors(obj):
    """
    finds all :class:`~django.db.models.fields.ForeignRelatedObjectsDescriptor` in obj.
    
    :param obj: A model instance or class.
    """
    return _get_members_of_type(obj, ForeignRelatedObjectsDescriptor)

def get_one_to_one_descriptors(obj):
    """
    finds all :class:`~django.db.models.fields.SingleRelatedObjectDescriptor` in obj.

    :param obj: A model instance or class.
    """
    return _get_members_of_type(obj, SingleRelatedObjectDescriptor)

def get_generic_relation_descriptors(obj):
    """
    Finds all the :class:`~django.contrib.contenttypes.generic.ReverseGenericRelatedObjectsDescriptor` in obj.
    
    :param obj: A model instance or class.
    """
    return _get_members_of_type(obj, ReverseGenericRelatedObjectsDescriptor)

def get_default_field_descriptors(obj):
    """
    find all  :class:`~easymode.i18n.meta.DefaultFieldDescriptor` in obj.
    
    :param obj: A model instance or class.
    """
    return _get_members_of_type(obj, DefaultFieldDescriptor)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Foo'
        db.create_table('foobar_foo', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('bar', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
            ('barstool', self.gf('django.db.models.fields.TextField')(max_length=4)),
            ('website', self.gf('django.db.models.fields.URLField')(max_length=200)),
            ('address', self.gf('django.db.models.fields.CharField')(max_length=32)),
            ('city_en-us', self.gf('django.db.models.fields.CharField')(max_length=40, null=True, blank=True)),
            ('city_de', self.gf('django.db.models.fields.CharField')(max_length=40, null=True, blank=True)),
            ('city_en', self.gf('django.db.models.fields.CharField')(max_length=40, null=True)),
            ('date_en-us', self.gf('django.db.models.fields.DateField')(null=True, blank=True)),
            ('date_de', self.gf('django.db.models.fields.DateField')(null=True, blank=True)),
            ('date_en', self.gf('django.db.models.fields.DateField')(null=True)),
        ))
        db.send_create_signal('foobar', ['Foo'])

        # Adding model 'Bar'
        db.create_table('foobar_bar', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('foo', self.gf('django.db.models.fields.related.ForeignKey')(related_name='bars', to=orm['foobar.Foo'])),
            ('label', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('date', self.gf('django.db.models.fields.DateField')()),
        ))
        db.send_create_signal('foobar', ['Bar'])

        # Adding model 'Baz'
        db.create_table('foobar_baz', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('baz', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('independancy_level', self.gf('django.db.models.fields.IntegerField')()),
            ('kind', self.gf('django.db.models.fields.CharField')(max_length=255)),
        ))
        db.send_create_signal('foobar', ['Baz'])


    def backwards(self, orm):
        
        # Deleting model 'Foo'
        db.delete_table('foobar_foo')

        # Deleting model 'Bar'
        db.delete_table('foobar_bar')

        # Deleting model 'Baz'
        db.delete_table('foobar_baz')


    models = {
        'foobar.bar': {
            'Meta': {'object_name': 'Bar'},
            'date': ('django.db.models.fields.DateField', [], {}),
            'foo': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'bars'", 'to': "orm['foobar.Foo']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'foobar.baz': {
            'Meta': {'object_name': 'Baz'},
            'baz': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'independancy_level': ('django.db.models.fields.IntegerField', [], {}),
            'kind': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'foobar.foo': {
            'Meta': {'object_name': 'Foo'},
            'address': ('django.db.models.fields.CharField', [], {'max_length': '32'}),
            'bar': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'barstool': ('django.db.models.fields.TextField', [], {'max_length': '4'}),
            'city_de': ('django.db.models.fields.CharField', [], {'max_length': '40', 'null': 'True', 'blank': 'True'}),
            'city_en': ('django.db.models.fields.CharField', [], {'max_length': '40', 'null': 'True'}),
            'city_en-us': ('django.db.models.fields.CharField', [], {'max_length': '40', 'null': 'True', 'blank': 'True'}),
            'date_de': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'date_en': ('django.db.models.fields.DateField', [], {'null': 'True'}),
            'date_en-us': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'website': ('django.db.models.fields.URLField', [], {'max_length': '200'})
        }
    }

    complete_apps = ['foobar']



"""
Lots of utility functions that can be used when
implementing a multi lingual django app.
"""
import re

from django.conf import settings
from django.utils.translation import get_language


USE_SHORT_LANGUAGE_CODES = getattr(settings, 'USE_SHORT_LANGUAGE_CODES', False)


def get_language_codes():
    """
    Retrieves all the language codes defined in ``settings.LANGUAGES``.
    
    >>> settings.LANGUAGES = (('en','English'),('de','German'),('nl-be','Belgium dutch'),('fr-be','Belgium french'),)
    >>> sorted( get_language_codes() )
    ['de', 'en', 'fr-be', 'nl-be']
    
    :rtype: A :class:`list` of language codes.
    """
    return dict(settings.LANGUAGES).keys()
    
def get_short_language_codes():
    """
    Retrieves the short versions of the language codes defined in settings.LANGUAGES.

    >>> from django.conf import settings
    >>> settings.LANGUAGES = (('en','English'),('de','German'),('nl-be','Belgium dutch'),('fr-be','Belgium french'),)
    >>> sorted( get_short_language_codes() )
    ['de', 'en', 'fr', 'nl']
    
    :rtype: A :class:`list` of short versions of the language codes.
    """
    languages = set([lang[:2] for lang in get_language_codes()])
    return list(languages)

# define regular expressions. they are different, depending on whether we are using full language
# codes, eg 'en-us' or we want to show the abbreviated versions in the url, eg. 'en'
if USE_SHORT_LANGUAGE_CODES:
    STRIP_LANGUAGE_CODE_REGEX = re.compile(ur'/(?:%s)/' % "|".join(get_short_language_codes()))
else:
    STRIP_LANGUAGE_CODE_REGEX = re.compile(ur'/(?:%s)/' % "|".join(get_language_codes()))

def get_all_language_codes():
    """
    Returns all language codes defined in settings.LANGUAGES and also the
    settings.MSGID_LANGUAGE if defined.
    
    >>> from django.conf import settings
    >>> settings.MSGID_LANGUAGE = 'en-us'
    >>> settings.LANGUAGES = (('en','English'),('de','German'),('nl-be','Belgium dutch'),('fr-be','Belgium french'),)
    >>> sorted( get_language_codes() )
    ['en-us', 'en','de', 'nl-be','fr-be']
    
    :rtype: A :class:`list` of language codes.
    """
    languages = get_language_codes()
    if hasattr(settings, 'MSGID_LANGUAGE'):
        if not settings.MSGID_LANGUAGE in languages:
            languages.insert(0, settings.MSGID_LANGUAGE)
            
    return languages

def get_shorthand_from_language_code(locale):
    """
    Returns the shorthand, based on the language code.
    
    If USE_SHORT_LANGUAGE_CODES is true it will return a
    ahorthand, if it is False, it will not modify the code
    at all.
    
    >>> USE_SHORT_LANGUAGE_CODES = True
    >>> get_shorthand_from_language_code('en-us')
    'en'
    >>> USE_SHORT_LANGUAGE_CODES = False
    >>> get_shorthand_from_language_code('en-us')
    'en-us'
    
    :param locale: The language code as a :class:`unicode` string.
    :rtype: The short version of the language code (when appropriate).
    """
    if USE_SHORT_LANGUAGE_CODES:
        return locale[:2]
        
    return locale

def get_language_code_from_shorthand(short_locale):
    """
    Returns the real language_code based on the shorthand.
    
    >>> settings.LANGUAGES = (('en-us','English'),('de','German'),('nl-be','Belgium dutch'),('fr-be','Belgium french'),)
    >>> get_language_code_from_shorthand('en')
    'en-us'
    
    :param short_locale: The short version of a language code.
    :rtype: The long version of the language code.
    """
    for locale in get_language_codes():
        if locale[:2] == short_locale:
            return locale
    return settings.LANGUAGE_CODE
    
def strip_language_code(url):
    """
    Strip the language code from the beginning of string

    >>> strip_language_code('http://example.com/en/example.html')
    'http://example.com/example.html'
    
    :param url: An url.
    :rtype: The ``url`` but the language code is removed.
    """
    return STRIP_LANGUAGE_CODE_REGEX.sub('/',url, 1)

def fix_language_code(url, current_language):
    """
    Fixes the language code as follows:
    
    If there is only one language used in the site, it strips the language code.
    If there are more languages used, it will make sure that the url has the current
    language as a prefix.
    
    >>> # multiple languages defined
    >>> settings.LANGUAGES = (('en-us','English'),('de','German'),('nl-be','Belgium dutch'),('fr-be','Belgium french'),)
    >>> settings.USE_SHORT_LANGUAGE_CODES = False
    >>> activate('en-us')
    >>> fix_language_code('/de/example.html', 'en-us')
    '/en-us/example.html'
    >>> settings.USE_SHORT_LANGUAGE_CODES = True
    >>> fix_language_code('/de/example.html', 'en-us')
    '/en/example.html'
    >>> fix_language_code('/en/example.html', 'en-us')
    '/example.html'
    >>> # only one language defined
    >>> settings.LANGUAGES = (('en-us', 'English'), )
    >>> fix_language_code('/en-us/example.html', 'en-us')
    '/example.html'
    
    :param url: The (absolute) url to be fixed eg. '/hi/how/is/it'.
    :param current_language: A language code defined in ``settings.LANGUAGES``.
    :rtype: The fixed url.
    """
    stripped_url = strip_language_code(url)
    if not getattr(settings, 'MASTER_SITE', False) and len(settings.LANGUAGES) == 1:
        # no MASTER_SITE and only one language, do not add language code to url
        return stripped_url
    
    # add the language code to the url        
    return u"/%s%s" % (get_shorthand_from_language_code(current_language), stripped_url)

def get_real_fieldname(field, lang):
    """
    Depending on the language a field can have a different name.
    
    If you want to do a query, using a localized field, you have
    to use the name this function returns, not the default name.
    
    >>> get_real_fieldname('name', 'en-us')
    'name_en-us'
    
    Usage::
    
        def index(request):
            published_in_language = get_real_fieldname('published', request.LANGUAGE_CODE)
            qs = SomeModel.objects.filter(**{published_in_language:True})
            return render_to_response('index.html', {'queryset':qs})
    
    
    :param field: The name of a field in an internationalised model.
    :param lang: The language for which you want to know the real name.
    :rtype: The actual name of the ``field`` in the ``lang``.
    """
    return str('%s_%s' % (field, lang))

def localize_fieldnames(fields, internationalized_fields):
    """
    Given a list of fields and a list of field names that
    are internationalized, will return a list with
    all internationalized fields properly localized.
    
    >>> from django.utils.translation import activate
    >>> activate('en-us')
    >>> localize_fieldnames(['name', 'title', 'url'], ['title'])
    ['name', 'title_en-us', 'url']
    
    :param fields: A :class:`list` af field names.
    :param internationalized_fields: A list of fields names, these fields are internationalized.
    :rtype: A list with the actual field names that are used in the current language.
    """
    result = []
    lang = get_language()    
    for field in fields:
        if field in internationalized_fields:
            result.append(get_real_fieldname(field, lang))
        else:
            result.append(field)    
    return result

def get_language_codes_as_disjunction():
    """
    return a pattern that matches any language defined in ``settings.LANGUAGES``
    
    >>> from django.conf import settings
    >>> settings.USE_SHORT_LANGUAGE_CODES = False
    >>> settings.LANGUAGES = (('en-us','English'),('de','German'),('nl-be','Belgium dutch'),('fr-be','Belgium french'),)
    >>> get_language_codes_as_disjunction()
    'de|en-us|nl-be|fr-be'
    
    usage::
        
        languages = get_language_codes_as_disjunction()
        urlpatterns = patterns('',
            url(r'^(%{languages})/admin/' % locals(), include(admin.site.urls), name="language-admin"),
        )
        
    """
    language_codes = map(get_shorthand_from_language_code, get_language_codes())
    languages = "|".join(language_codes)
    return languages



"""
Contains formsets where the add_fields method does not hide 
the autofield.
"""
from django.forms.models import BaseModelFormSet
from django.forms import IntegerField
from django.forms.formsets import BaseFormSet
from django.utils.translation import ugettext_lazy as _

from easymode.tree.admin.widgets.foreignkey import RenderLink

class VisiblePrimaryKeyFormset(BaseModelFormSet):
    """
    This formset will show the primary keys instead of hide it.
    """
    def add_fields(self, form, index):

        self._pk_field = self.model._meta.pk

        if form.is_bound:
            pk_value = form.instance.pk
        else:
            try:
                pk_value = self.get_queryset()[index].pk
            except IndexError:
                pk_value = None

        attrs = dict(app_label=self.model._meta.app_label, modelname=self.model.__name__.lower(), label=unicode(form.instance))
        form.fields[self._pk_field.name] = IntegerField( initial=pk_value, required=False, 
                                                         widget=RenderLink(attrs=attrs), label=_("Navigate to:"))

        BaseFormSet.add_fields(self, form, index)



from django.forms.models import ModelFormMetaclass
from django.forms.util import ErrorList, ValidationError
from django.utils.datastructures import SortedDict
from django.utils.translation import get_language

from easymode.i18n.admin.widgets import WidgetWrapper
from easymode.utils.languagecode import get_real_fieldname


__all__ = ('make_localised_form',)


class LocalisedForm(object):
    """
    This form will show the DefaultFieldDescriptor as a field.
    The DefaultFieldDescriptor is added for models that are internationalised
    with the I18n descriptor.
    """
        
    def __init__(self, data=None, files=None, auto_id='id_%s', prefix=None,
                 initial=None, error_class=ErrorList, label_suffix=':',
                 empty_permitted=False, instance=None, js=None):
        
        # store language so it can be used later
        self.language = get_language()
        
        locale_data = initial or SortedDict()

        # Set up the initial data for the form
        for localized_field in self.localized_fields:
            # determine localized name of the field, because it is called like
            # that in the initial dict
            local_name = get_real_fieldname(localized_field, self.language)

            if initial: # get value from initial if it is defined
                initial_value = initial.get(local_name)
                if not initial_value and instance: # try model if defined
                    initial_value = getattr(instance, localized_field)
                locale_data[localized_field] = initial_value
            elif instance is not None:
                locale_data[localized_field] = getattr(instance, localized_field)
            
        super(LocalisedForm, self).__init__(data, files, auto_id, prefix,
                                            locale_data, error_class, 
                                            label_suffix, empty_permitted, 
                                            instance)
                                              
    def save(self, commit=True):
        """
        Override save method to also save the localised fields.
        """
        # set the localised fields
        for localized_field in self.instance.localized_fields:
            setattr(self.instance, localized_field, self.cleaned_data[localized_field])

        return super(LocalisedForm, self).save(commit)

    def validate_unique(self):
        """
        Validates the uniqueness of fields, but also handles the localized_fields.
        """
        form_errors = []
        try:
            super(LocalisedForm, self).validate_unique()
        except ValidationError as e:
            form_errors += e.messages

        # add unique validation for the localized fields.
        localized_fields_checks = self._get_localized_field_checks()
        bad_fields = set()

        field_errors, global_errors = self._perform_unique_localized_field_checks(localized_fields_checks)
        bad_fields.union(field_errors)
        form_errors.extend(global_errors)
        
        for field_name in bad_fields:
            del self.cleaned_data[field_name]
        if form_errors:
            # Raise the unique together errors since they are considered
            # form-wide.
            raise ValidationError(form_errors)
    
    def _get_localized_field_checks(self):
        """
        Get the checks we must perform for the localized fields.
        """
        localized_fields_checks = []
        for localized_field in self.instance.localized_fields:
            if self.cleaned_data.get(localized_field) is None:
                continue
            f = getattr(self.instance.__class__, localized_field, None)
            if f and f.unique:
                if f.unique:
                    local_name = get_real_fieldname(localized_field, self.language)
                    localized_fields_checks.append((localized_field, local_name))
                    
        return localized_fields_checks

    def _perform_unique_localized_field_checks(self, unique_checks):
        """
        Do the checks for the localized fields.
        """
        bad_fields = set()
        form_errors = []

        for (field_name, local_field_name) in unique_checks:
            
            lookup_kwargs = {}
            lookup_value = self.cleaned_data[field_name]
            # ModelChoiceField will return an object instance rather than
            # a raw primary key value, so convert it to a pk value before
            # using it in a lookup.
            lookup_value = getattr(lookup_value, 'pk', lookup_value)
            lookup_kwargs[str(local_field_name)] = lookup_value

            qs = self.instance.__class__._default_manager.filter(**lookup_kwargs)

            # Exclude the current object from the query if we are editing an
            # instance (as opposed to creating a new one)
            if self.instance.pk is not None:
                qs = qs.exclude(pk=self.instance.pk)

            # This cute trick with extra/values is the most efficient way to
            # tell if a particular query returns any results.
            if qs.extra(select={'a': 1}).values('a').order_by():
                self._errors[field_name] = ErrorList([self.unique_error_message([field_name])])
                bad_fields.add(field_name)
                
        return bad_fields, form_errors

def make_localised_form(model, form, exclude=None):
    """
    This is a factory function that creates a form for a model with internationalised 
    field. The model should be decorated with the L10N decorater.
    """
    
    newfields = {}  
    
    for localized_field in model.localized_fields:
        # get the descriptor, which contains the form field
        default_field_descriptor = getattr(model, localized_field)
        
        # See if we've got overridden fields in a custom form.
        if hasattr(form, 'declared_fields'):
            form_field = form.declared_fields.get(
                localized_field, default_field_descriptor.form_field)
        else:
            form_field = default_field_descriptor.form_field
         
        # wrap the widget to show the origin of the value;
        # either database, catalog or fallback.
        if type(form_field.widget) is not WidgetWrapper:
            form_field.widget = WidgetWrapper(form_field.widget)
         
        newfields[localized_field] = form_field
 
    if hasattr(form, 'Meta'):
         setattr(form.Meta, 'model', model)
    else:
         newfields['Meta'] = type('Meta', tuple(), {'model':model})
    
    newfields['localized_fields'] = model.localized_fields

    return ModelFormMetaclass(model.__name__, (LocalisedForm, form), newfields)



"""
Contains a decorator that can be used to add xml serialization to django
models as a method. In effect it adds an ``__xml__`` method to both the model and
the queryset used in the model.
"""

from easymode.tree.serializers import RecursiveXmlSerializer
from easymode.tree.query import QuerySetManager, XmlSerializableQuerySet

def toxml(cls):
    """
    adds an ``__xml__`` method to both the queryset as the model class.
    
    usage::
    
        from easymode.tree.decorators import toxml

        @toxml
        class Foo(models.ModelAdmin):
            title = models.CharField(max_length=255)
            content = TextField()

        class Bar(models.ModelAdmin):
            foo = models.ForeignKey(Foo, related_name=bars)

            label = models.CharField(233)

    The ``Foo`` model has now gained a ``__xml__`` method on both itself as on the
    queryset it produces. Calling it will produce hierarchical xml, where all inverse
    relations are followed. (Except for manytomany fields, they are not supported)."""
    
    if cls._meta.abstract:
        raise Exception("You can not use toxml on abstract classes")
        
    def __xml__(self):
        """turn model object into xml recursively"""
        ser = RecursiveXmlSerializer()
        return ser.serialize([self])
    
    cls.add_to_class('objects', QuerySetManager(XmlSerializableQuerySet))
    cls.__xml__ = __xml__
    return cls



from django.utils.translation import ugettext_lazy as _
from django.db import models

class AbstractOrderedModel(models.Model):
    """
    This model includes functionality for ordering.
    """
    
    order = models.PositiveIntegerField(_('Order'), default=99999999)
    
    class Meta:
        abstract = True
        ordering = ('order',)


from types import NoneType
from django.utils.text import capfirst

__all__ = ('standin_for',)

_defined_standins = dict()
    
def standin_for(obj, **attrs):
    """
    Returns an object that can be used as a standin for the original object.
    
    The standin object will have extra attrs, which you can define passed as keyword arguments.
    
    Use standin like this:
    
    >>> a = u'I am E.T.'
    >>> b = standin_for(a, origin='outerspace', package='easymode.utils.standin')
    >>> b
    u'I am E.T.'
    >>> b == a
    True
    >>> b.origin
    'outerspace'
    >>> b.package
    'easymode.utils.standin'
    >>> isinstance(b, unicode)
    True
    >>> type(b)
    <class 'easymode.utils.standin.unicodeStandInWithOriginAndPackageAttributes'>
    >>> import pickle
    >>> b_pickle = pickle.dumps(b, pickle.HIGHEST_PROTOCOL)
    >>> c = pickle.loads(b_pickle)
    >>> b == c
    True
    >>> c
    u'I am E.T.'
    >>> c.origin
    'outerspace'
    >>> c.package
    'easymode.utils.standin'
    
    Some types are not supported by standin_for. This is because these types are often used in statements like::
        
        a = True
        if a is True:
            print 'hi'
    
    The *is* keyword checks for equal memory adress, and in case of a standin that can never be *True*.
    Also, since it is impossible to extend :class:`bool` and :class:`~types.NoneType` you can never get::
    
        isinstance(standin, bool)
        isinstance(standin, NoneType)
    
    To work with anything else but the real thing.
    This is why :class:`bool` and :class:`~types.NoneType` instances are returned unmodified:
    
    >>> a = False
    >>> b = standin_for(a, crazy=True)
    >>> b
    False
    >>> b.crazy
    Traceback (most recent call last):
        ...
    AttributeError: 'bool' object has no attribute 'crazy'
        
    :param obj: An instance of some class
    :param **attrs: Attributes that will be added to the standin for *obj*
    :rtype: A new object that can be used where the original was used. However it has extra attributes.
    """
    
    obj_class = obj.__class__
    if obj_class is bool or obj_class is NoneType:
        # we can not have a standing for bool or NoneType
        # because too many code uses a is True or a is None.
        # Also you can never get isinstance(standin, bool) and
        # isinstance(standin, NoneType) to work because you can
        # never extend these types.
        return obj
    
    attr_names  = attrs.keys()
    attr_names.sort()

    # Contruct __reduce__ method, so the resulting class can be pickled
    attrs_org = attrs.copy() # Create a copy to be used for the recude method
    def __reduce__(self, ignore=None):
        return (_standin_with_dict_for, (obj, attrs_org))
    attrs['__reduce__'] = __reduce__
    attrs['__reduce_ex__']= __reduce__

    # create a readable class name eg. unicodeStandinWithTitleAndDescriptionAttributes
    additions = 'And'.join(map(capfirst, attr_names))
    id = "%sStandInWith%sAttributes" % (obj_class.__name__, additions.encode('ascii', 'ignore'))
    
    # if we allready know this type don't create it again.
    cached_type = _defined_standins.get(id, None)
    if not cached_type:
        cls_attrs = dict([(attr_name, None) for attr_name in attr_names])
        cached_type = type(id, (obj_class,), cls_attrs)
        _defined_standins[id] = cached_type
    
    # create new object based on original and copy all properties
    try:
        stand_in = cached_type(obj)
    except (AttributeError, TypeError):
        try:
            stand_in = cached_type()
            stand_in.__dict__.update(obj.__dict__)
        except (AttributeError, TypeError):
            stand_in = obj

    # add extra attrs
    try:
        for (key, value) in attrs.iteritems():
            setattr(stand_in, key, value)
    except AttributeError:
        # if nothing works return original object
        return obj
    
    return stand_in
    
def _standin_with_dict_for(obj, attrs):
    return standin_for(obj, **attrs)


import warnings
from django.conf import settings
from django.test import TestCase
from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.translation import get_language
from django.contrib.auth.models import User, Permission

from easymode.utils.languagecode import get_real_fieldname
from easymode.tests.models import TestL10nModel
from easymode.tests.testcases import initdb

from easymode.easypublisher.models import EasyPublisherMetaData

import reversion

if not 'easymode.easypublisher' in settings.INSTALLED_APPS:
    raise Exception('''Can not run easypublisher tests without easymode.easypublisher in INSTALLED_APPS.''')
if not 'reversion' in settings.INSTALLED_APPS:
    raise Exception('''Can not run easypublisher tests without reversion in INSTALLED_APPS''')

__all__ = ('TestEasyPublisher', 'TestEasyPublisherInline')

@initdb
class TestEasyPublisher(TestCase):
    """Tests for easypublisher"""
    fixtures = ['auth-user', 'auth-group', 'tests-testl10nmodel']
    
    post_data = {
        'price': 100,
        'title': 'Your mother',
        'description': 'lmao',
        'body': 'MY BAR IS LOL HAHA',
    }
    
    username = 'editor'
    password = 'editor'
    
    def test_login_as_non_privileged_user(self):
        "It should be possible to log in as editor"
        self.assertTrue( self.client.login(username=self.username, password=self.password) )
    
    def test_login_and_post(self):
        "An editor should be able to post to the change_view of TestL10nModel"
        self.test_login_as_non_privileged_user()
                
        admin_url = reverse('admin:tests_testl10nmodel_change', args=[1])
        response = self.client.post(admin_url, self.post_data)
        self.failUnlessEqual(response.status_code, 302)
        return response
    
    def test_non_privileged_user_can_not_change_model(self):
        "All changes done by an editor to TestL10nModel should not be saved to the model table"
        self.test_login_and_post()
        
        # the contents of the first TestL10nModel should not be changed
        data = TestL10nModel.objects.get(pk=1)
        self.failIfEqual(data.price, self.post_data['price'])
        self.failIfEqual(data.title, self.post_data['title'])
        self.failIfEqual(data.description, self.post_data['description'])
        self.failIfEqual(data.body, self.post_data['body'])
    
    def test_only_one_easy_publisher_meta_data_was_created(self):
        "For each draft created by an editor, an EasyPublisherMetaData object should be created"
        self.test_non_privileged_user_can_not_change_model()
        
        # there should be a test in reversion
        metas = EasyPublisherMetaData.objects.select_related()
        self.failUnlessEqual(len(metas), 1)
    
        return metas[0]
        
    def test_reversion_has_the_draft(self):
        "A data belonging to a draft should be in reversion as a version"
        meta = self.test_only_one_easy_publisher_meta_data_was_created()
        
        for version in meta.revision.version_set.all():
            obj = version.get_field_dict()
            self.failUnlessEqual(obj[get_real_fieldname('title', get_language())], self.post_data['title'])
            self.failUnlessEqual(obj[get_real_fieldname('description', get_language())], self.post_data['description'])
        
    def test_publication_workflow(self):
        "After the draft is saved/published, the data should be in the model table"
        
        meta = self.test_only_one_easy_publisher_meta_data_was_created()
        
        # restoring the item will save the shizz
        for version in meta.revision.version_set.all():
            version.revert()
        
        # and now the database should reflect the post of the editor
        data = TestL10nModel.objects.get(pk=1)
        self.failUnlessEqual(data.title, self.post_data['title'])
        self.failUnlessEqual(data.description, self.post_data['description'])
        return data
    
    def draft_access_helper(self):
        admin_url = reverse('admin:tests_testl10nmodel_change', args=[1])
        response = self.client.get(admin_url)
        self.failUnlessEqual(response.status_code, 302)

        # See that the user is redirected to the draft
        draft_url = reverse('admin:tests_testl10nmodel_draft', args=[1, 1])
        self.assertRedirects(response, draft_url)
        
    def test_non_privileged_users_get_drafts_when_accessing_items(self):
        """
        When a non privileged user tries to access an item with a draft,
        instead the draft itself should be presented
        """
        # create a draft
        self.test_publication_workflow()
        self.draft_access_helper()
        
    def test_privileged_users_get_drafts_when_accessing_items(self):
        self.test_publication_workflow()
        self.username = 'admin'
        self.password = 'admin'
        self.draft_access_helper()
        self.username = 'editor'
        self.password = 'editor'
    
    def test_draft_view(self):
        self.test_publication_workflow()
        draft_url = reverse('admin:tests_testl10nmodel_draft', args=[1, 1])
        
        # get draft
        response = self.client.get(draft_url)
        self.assertContains(response, 'Press the save button below to make this draft the current version')
        self.assertContains(response, '<a href="../../current/" class="historylink">Current</a>')
        self.assertContains(response, '<a href="../../drafts/" class="historylink">Drafts</a>')
        self.assertTemplateUsed(response, 'easymode/easypublisher/publish_form.html')
    
    def test_current_view(self):
        current_url = reverse('admin:tests_testl10nmodel_current', args=[1])
        
    def test_non_privileged_user_can_not_edit_restricted_fields_without_permission(self):
        """If a user does not have the can_edit_global_fields permission for a
        model, untranslated fields should be hidden and not editable in any way"""
        
        self.client.login(username=self.username, password=self.username)
        
        admin_url = reverse('admin:tests_testl10nmodel_change', args=[1])
        response = self.client.get(admin_url)
        self.failUnlessEqual(response.status_code, 200)
        
        # the body and price fields should be excluded for this user
        self.assertNotContains(response, '<textarea rows="10" cols="40" name="body" id="id_body">hi i am an error</textarea>')
        self.assertNotContains(response, '<input type="text" name="price" value="200.0" id="id_price" />')
        
        data = self.test_publication_workflow()
        self.failIfEqual(data.price, self.post_data['price'])
        self.failIfEqual(data.body, self.post_data['body'])
    
    def test_if_editor_has_permission_he_can_edit_all_fields(self):
        """If a user has permision can_edit_untranslated_fields_of_testl10nmodel
        he should be able to edit all the untranslated fields"""
        
        # give the editor permission
        editor = User.objects.get(username=self.username)
        perm = Permission.objects.get(codename='can_edit_untranslated_fields_of_testl10nmodel')
        editor.user_permissions.add(perm)
        editor.save()
    
        data = self.test_publication_workflow()
        self.failUnlessEqual(data.price, self.post_data['price'])
        self.failUnlessEqual(data.body, self.post_data['body'])

@initdb
class TestEasyPublisherInline(TestCase):
    fixtures = ['auth-user', 'auth-group', 'tests-testeasypublisher']
    
    def test_admin_can_approve(self):
        pass